{---------------------------------------------------------------}
{
{    Easy Board - Editor and Presenter of
{                    Boards for alternative communication
{
{    Module of archives treatment
{
{    Authors: Jos Antonio Borges     (NCE/UFRJ)
{             Miryam Bonadiu Pelosi    (LabAssistiva/UFRJ)
{             Julio Tadeu da Silveira  (NCE/UFRJ)
{
{    Version 1.1. in april/2015
{
{----------------------------------------------------------------}

unit prArq;

interface
uses
  Consts,     // customized version to English
  windows, graphics, prvars, sysutils, classes, dialogs, typInfo;

procedure carregaPranchas (dirPranchas: string);
procedure salvaPranchas (dirPranchas, nomeNovo: string);
procedure adicionaArquivos (sl: TStrings; extensao: string);

implementation

function infoGet (nomeArq, titulo, secao: string): string;
var
    returnString: array[0..255] of char;
    s: string;
begin
    if secao = '' then
        s := ''
    else
        begin
            GetPrivateProfileString(pchar(titulo), pchar(secao),
                 '', returnString, 255, pchar(nomeArq));
            s := strPas(returnString);
        end;
    infoGet := trim(s);
end;

{------------------------------------------------------------------------}

procedure carregaPranchas (dirPranchas: string);
const
    MAXTAMLISTA = 50000;
var
    lista: array [0..MAXTAMLISTA-1] of char;
    p, i, npr: integer;
    l: pchar;
    nomeArq: string;
    modelo: string;
    s, si: string;
    erro: integer;

    function pegaTamanhoTexto (s: string): integer;
    begin
        s := trim (s);
        if uppercase(s) = 'SMALL'   then result := TEXTO_PEQUENO
        else
        if (uppercase(s) = 'NORMAL') or
           (uppercase(s) = 'NORMAL')   then result := TEXTO_MEDIO
        else
        if uppercase(s) = 'BIG'    then result := TEXTO_GRANDE
        else
            try
                 result := strToInt(s);
            except
                 result := TEXTO_MEDIO;
            end;
    end;

begin
    for p := 0 to projeto.nPranchas-1 do
        with projeto.pranchas[p] do
              SetLength(itens, 0);

    setLength(projeto.pranchas, 0);
    projeto.nPranchas := 0;

    nomeArq := dirPranchas + '\pranchas.pra';
    if not FileExists(nomeArq) then
        begin
            showMessage ('File not found');
            exit;
        end;

    getPrivateProfileString(NIL, NIL, '', lista, MAXTAMLISTA, pchar(nomeArq));

    // Varredura para calcular o tamanho para alocar tudo de uma vez
    npr := 0;
    l := lista;
    while l^ <> #0 do
        begin
            npr := npr + 1;
            l := l + strLen (l) + 1;
        end;

    setLength(projeto.pranchas, npr);
    projeto.nPranchas := npr;

    with projeto do
        begin
            nome := 'Boards';
            autor := 'UFRJ';
            data := dateTimeToStr(now);
            info := 'Easy Board' + versao;
        end;

    l := lista;
    if strPas(l) = 'CONFIG' then
        begin
            npr := npr - 1;
            projeto.nPranchas := npr;
            l := l + strLen (l) + 1;

            with projeto do
                begin
                    autor := infoGet (nomeArq, 'CONFIG', 'AUTOR');
                    if autor = '' then autor := 'UFRJ';
                    data := infoGet (nomeArq, 'CONFIG', 'DATA');
                    if data = '' then
                        data := DateTimeToStr(now);

                    s := infoGet (nomeArq, 'CONFIG', 'FALANDO');
                    falaAutomatica := upperCase (copy (s, 1, 1)) = 'S';
                    s := infoGet (nomeArq, 'CONFIG', 'FALAVARREDURA');
                    falaVarredura := upperCase (copy (s, 1, 1)) = 'S';

                    info := infoGet (nomeArq, 'CONFIG', 'INFO');
                    if info = '' then
                        info := 'Prancha Fcil ' + versao;

                    s := infoGet (nomeArq, 'CONFIG', 'SINTETIZANDO');
                    SintetNaoGravados := upperCase (copy (s, 1, 1)) = 'S';
                    s := infoGet (nomeArq, 'CONFIG', 'VARREDURA');
                    val (s, tempoVarredura, erro);
                    if erro <> 0 then tempoVarredura := 2000;
                    s := infoGet (nomeArq, 'CONFIG', 'HISTERESE');
                    val (s, tempoHisterese, erro);
                    if erro <> 0 then tempoHisterese := 1000;
                    s := infoGet (nomeArq, 'CONFIG', 'AUTOCLIQUE');
                    val (s, tempoAutoClique, erro);
                    if erro <> 0 then tempoAutoClique := 0;
                    s := infoGet (nomeArq, 'CONFIG', 'SOCORRO');
                    val (s, tempoSocorro, erro);
                    if erro <> 0 then tempoSocorro := 3000;
                    pranchaInicial := infoGet (nomeArq, 'CONFIG', 'PRANCHA_INICIAL');
                    pranchaSocorro := infoGet (nomeArq, 'CONFIG', 'PRANCHA_SOCORRO');
                end;
        end;

    for p := 0 to npr-1 do
        with projeto.pranchas[p] do
            begin
                nome := strPas (l);
                modelo := infoGet (nomeArq, nome, 'MODELO');
                nFigy := strtoInt (copy (modelo, 1, 1));  // melhorar
                nFigx := strtoInt (copy (modelo, 3, 1));
                nitens := nFigx * nFigy;
                titulo := infoGet (nomeArq, nome, 'TITULO');
                linkDefault := infoGet (nomeArq, nome, 'LINK');
                s := infoGet (nomeArq, nome, 'COR');
                if s = '' then
                    cor := clWhite
                else
                    cor := stringToColor (s);
                som := infoGet (nomeArq, nome, 'SOM');
                fala:= infoGet (nomeArq, nome, 'FALA');

                setLength(itens, nitens);
                for i := 1 to nitens do
                    with itens[i-1] do
                        begin
                            si := intToStr(i);
                            rotulo := infoGet (nomeArq, nome, si);
                            figura := infoGet (nomeArq, nome, 'FIGURA '+si);
                            link := infoGet (nomeArq, nome, 'LINK '+si);
                            s := infoGet (nomeArq, nome, 'COR '+si);
                            if s = '' then
                                cor := clWhite
                            else
                                cor := stringToColor (s);
                            som := infoGet (nomeArq, nome, 'SOM '+si);
                            fala := infoGet (nomeArq, nome, 'FALA '+si);
                            executar := infoGet (nomeArq, nome, 'EXECUTAR '+si);

                            s := infoGet (nomeArq, nome, 'POSTEXTO '+si);
                            if s = '' then
                                posTexto := baixo
                            else
                                posTexto := TPosVertical(getEnumValue(TypeInfo(TPosVertical), s));

                            s := infoGet (nomeArq, nome, 'DIMTEXTO '+si);
                            if s = '' then
                                dimTexto := TEXTO_MEDIO
                            else
                                dimTexto := pegaTamanhoTexto(s);

                            s := infoGet (nomeArq, nome, 'DIMMOLDURA '+si);
                            if s = '' then
                                dimMoldura := sem 
                            else
                                dimMoldura := TDimMoldura(getEnumValue(TypeInfo(TDimMoldura), s));

                            s := infoGet (nomeArq, nome, 'SEMROTULO '+si);
                            semRotulo := copy (s, 1, 1) = 'S';
                            s := infoGet (nomeArq, nome, 'SEMFIGURA '+si);
                            semFigura := copy (s, 1, 1) = 'S';
                            s := infoGet (nomeArq, nome, 'DESLOCHORIZ '+si);
                            deslocHoriz := copy (s, 1, 1) = 'S';
                        end;

                l := l + strLen (l) + 1;
            end;
end;

{------------------------------------------------------------------------}

procedure salvaPranchas (dirPranchas, nomeNovo: string);
var
    nomeArq: string;
    arq: textFile;
    i, p: integer;
const
    boolString: array [boolean] of string = ('NO', 'SIM');
begin
    nomeArq := dirPranchas + '\' + nomeNovo;
    assignFile (arq, nomeArq);
    {$I-} rewrite (arq); {$I+}
    if ioresult <> 0 then
        begin
            showMessage ('BOARDS FILE COULD NOT BE SAVED');
            exit;
        end;

    writeln (arq, ';-------------------------------------------------------------------------');
    writeln (arq, ';  Prancha Fcil');
    writeln (arq, ';  ' + projeto.nome);
    writeln (arq, ';  Autor: ' + projeto.autor);
    writeln (arq, ';  Em ' + projeto.data);
    writeln (arq, ';-------------------------------------------------------------------------');
    writeln (arq);
    writeln (arq, '[CONFIG]');
    writeln (arq, 'AUTOR=' + projeto.autor);
    writeln (arq, 'DATA=' + projeto.data);
    writeln (arq, 'FALANDO=' + boolString[projeto.falaAutomatica]);
    writeln (arq, 'SINTETIZANDO=' + boolString[projeto.sintetNaoGravados]);
    writeln (arq, 'FALAVARREDURA' + boolString[projeto.falaVarredura]);
    writeln (arq, 'VARREDURA=' + intToStr(projeto.tempoVarredura));
    writeln (arq, 'HISTERESE=' + intToStr(projeto.tempoHisterese));
    writeln (arq, 'AUTOCLIQUE=' + intToStr(projeto.tempoAutoClique));
    writeln (arq, 'SOCORRO=' + intToStr(projeto.tempoSocorro));
    writeln (arq, 'PRANCHA_INICIAL=' + projeto.pranchaInicial);
    writeln (arq, 'PRANCHA_SOCORRO=' + projeto.pranchaSocorro);
    writeln (arq);
    writeln (arq, '; FALANDO=SIM indica que sero lidos os rtulos, com o sintetizador do Windows');
    writeln (arq, '; VARREDURA=0 indica sem varredura');
    writeln (arq, '; HISTERESE=0 indica que a pessoa pode dar vrios cliques em sequncia');
    writeln (arq, '; SOCORRO=0 indica que o tempo de desvio para a prancha ESCAPE no  usado');

    for p := 0 to projeto.npranchas-1 do
        with projeto.pranchas[p] do
            begin
                writeln (arq);
                writeln (arq, ';------------------------------------------');
                writeln (arq);
                writeln (arq, '[' + nome + ']');
                writeln (arq, 'MODELO=', nFigy, 'X', nFigx);
                writeln (arq, 'TITULO=', titulo);
                if linkDefault  <> '' then
                    writeln (arq, 'LINK=', linkDefault);
                if cor <> clWhite then
                    writeln (arq, 'COR=', cor);
                if som <> '' then
                    writeln (arq, 'SOM=', som);
                if fala <> '' then
                    writeln (arq, 'FALA=', fala);

                writeln (arq);
                for i := 1 to nitens do
                    with itens[i-1] do
                        begin
                            if rotulo = '' then continue;

                            writeln (arq, i, '=', rotulo);
                            if figura <> '' then
                                writeln (arq, '    FIGURA ', i, '=', figura);
                            if link <> '' then
                                writeln (arq, '    LINK ', i, '=', link);
                            if cor <> clWhite then
                                writeln (arq, '    COR ', i, '=', colorToString(cor));
                            if som <> '' then
                                writeln (arq, '    SOM ', i, '=', som);
                            if fala <> '' then
                                writeln (arq, '    FALA ', i, '=', fala);
                            if executar <> '' then
                                writeln (arq, '    EXECUTAR ', i, '=', executar);

                            if posTexto <> baixo then
                                writeln (arq, '    POSTEXTO ', i, '=',
                                     GetEnumName(TypeInfo(TPosVertical),  integer(posTexto)));
                            if dimTexto <> TEXTO_MEDIO then
                                writeln (arq, '    DIMTEXTO ', i, '=', intToStr(dimTexto));
                            if dimMoldura <> sem then
                                writeln (arq, '    DIMMOLDURA ', i, '=',
                                     GetEnumName(TypeInfo(TDimMoldura),  integer(dimMoldura)));
                            if deslocHoriz then
                                writeln (arq, '    DESLOCHORIZ ', i, '=SIM');
                            if semRotulo then
                                writeln (arq, '    SEMROTULO ', i, '=SIM');
                            if semFigura then
                                writeln (arq, '    SEMFIGURA ', i, '=SIM');
                        end;
            end;

    closeFile (arq);

    showMessage ('Boards saved');
end;

{------------------------------------------------------------------------}

procedure adicionaArquivos (sl: TStrings; extensao: string);
var FileAttrs: integer;
    sr: TSearchRec;
begin
    fileAttrs := faArchive;
    if FindFirst(dirProjeto+'\*.' + extensao, FileAttrs, sr) = 0 then
        begin
            repeat
                if (sr.Attr and FileAttrs) <> 0 then
                    sl.add (sr.Name);
            until FindNext(sr) <> 0;
            FindClose(sr);
        end;
end;

begin
    projeto.nPranchas := 0;
end.

