{---------------------------------------------------------------}
{
{    Easy Board - Editor and Presenter of
{                 Boards for alternative communication
{
{    Module: Board configuration
{
{    Authors: Jos Antonio Borges     (NCE/UFRJ)
{             Miryam Bonadiu Pelosi    (LabAssistiva/UFRJ)
{             Julio Tadeu da Silveira  (NCE/UFRJ)
{
{    Version 1.1 in april/2015
{
{----------------------------------------------------------------}

unit prConfigPrancha;

interface

uses
  Consts,     // customized version to English
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, jpeg, ExtCtrls, Spin, StdCtrls, Buttons;

type
  TformConfigPrancha = class(TForm)
    Panel1: TPanel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label8: TLabel;
    Label7: TLabel;
    Shape1: TShape;
    e_nome: TEdit;
    e_ny: TSpinEdit;
    e_nx: TSpinEdit;
    b_importarSom: TButton;
    ColorDialog1: TColorDialog;
    Label1: TLabel;
    e_titulo: TEdit;
    combo_som: TComboBox;
    combo_link: TComboBox;
    OpenDialog1: TOpenDialog;
    p_prancha: TPanel;
    Label9: TLabel;
    Label10: TLabel;
    b_play: TBitBtn;
    b_stop: TBitBtn;
    b_record: TBitBtn;
    b_ok: TButton;
    Label11: TLabel;
    b_playFala: TBitBtn;
    b_stopFala: TBitBtn;
    b_recFala: TBitBtn;
    combo_fala: TComboBox;
    procedure b_importarSomClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure e_tituloChange(Sender: TObject);
    procedure e_dimChange(Sender: TObject);
    procedure Shape1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure b_playClick(Sender: TObject);
    procedure b_stopClick(Sender: TObject);
    procedure b_recClick(Sender: TObject);
    procedure b_stopFalaClick(Sender: TObject);
    procedure b_playFalaClick(Sender: TObject);
    procedure b_recFalaClick(Sender: TObject);
    procedure b_okClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure setInfo (ind: integer);
    procedure getInfo (ind: integer);
    procedure desenhaPrancha (ind: integer);
  end;

var
  formConfigPrancha: TformConfigPrancha;

implementation

uses prVars, prArq, prGraficos, prGravador, prMci;

var
    pranchaGraf: TPranchaGraf;
    indPranchaAtual: integer;

{$R *.dfm}

procedure TformConfigPrancha.setInfo (ind: integer);
begin
    if ind < 0 then exit;
    with projeto.pranchas[ind] do
        begin
            indPranchaAtual := -1;

            e_nome.Text := nome;
            e_titulo.Text := titulo;
            e_nx.Text := intToStr (nFigx);
            e_ny.Text := intToStr (nFigy);
            combo_fala.Items.Clear;
            combo_fala.Text := fala;
            combo_som.Items.Clear;
            combo_som.Text := som;
            combo_link.Text := linkDefault;
            shape1.Brush.Color := cor;

            indPranchaAtual := ind;
        end;
end;

procedure TformConfigPrancha.getInfo (ind: integer);
begin
    if ind < 0 then exit;
    with projeto.pranchas[ind] do
        begin
            nome := e_nome.Text;
            titulo := e_titulo.Text;
            nfigX := strToInt (e_nx.Text);
            nfigY := strToInt (e_ny.Text);
            fala := combo_fala.Text;
            som := combo_som.Text;
            linkDefault := combo_link.Text;
            cor := shape1.Brush.Color;
        end;
end;

procedure TformConfigPrancha.desenhaPrancha (ind: integer);
var
    nx, ny: integer;
    erro1, erro2: integer;
begin
    if ind < 0 then exit;
    val (e_nx.text, nx, erro1);
    val (e_ny.text, ny, erro2);
    if erro1+erro2 <> 0 then exit;

    with projeto.pranchas[ind] do
        criaPranchaGraf (pranchaGraf, e_titulo.text, nx, ny, shape1.Brush.Color,
                         projeto.pranchas[ind].itens,
                         p_prancha);
end;

procedure TformConfigPrancha.FormActivate(Sender: TObject);
var p: integer;
begin
    combo_som.items.Clear;
    combo_som.Sorted := false;
    adicionaArquivos (combo_som.items, 'mp3');
    adicionaArquivos (combo_som.items, 'wav');
    combo_som.Sorted := true;
    combo_fala.Items.Assign(combo_som.Items);

    combo_link.Items.clear;
    for p := 0 to projeto.nPranchas-1 do
        combo_link.Items.add (projeto.pranchas[p].nome);

    desenhaPrancha (indPranchaAtual);
end;

procedure TformConfigPrancha.b_importarSomClick(Sender: TObject);
var nomeSom: string;
begin
    openDialog1.InitialDir := dirProjeto;
    openDialog1.Filter := '*.mp3|*.mp3|*.wav|*.wav|*.*|*.*';
    if openDialog1.Execute then
        begin
            nomeSom := ExtractFileName(openDialog1.FileName);
            combo_som.Text := nomeSom;
            copyFile (pchar(openDialog1.FileName), pchar(dirProjeto+'\'+nomeSom), false);
        end;

    chdir (dirProjeto);
end;

procedure TformConfigPrancha.e_dimChange(Sender: TObject);
begin
    destroiPranchaGraf (pranchaGraf);
    desenhaPrancha (indPranchaAtual);
end;

procedure TformConfigPrancha.e_tituloChange(Sender: TObject);
begin
     desenhaPrancha (indPranchaAtual);
end;

procedure TformConfigPrancha.Shape1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if ColorDialog1.Execute then
         begin
             shape1.Brush.Color := ColorDialog1.Color;
             desenhaPrancha(indPranchaAtual);
         end;
end;

procedure TformConfigPrancha.b_playClick(Sender: TObject);
var nomeSom: string;
begin
    nomeSom := combo_som.text;
    if (nomeSom <> '') and (copy (nomeSom, length(nomeSom)-3, 1) <> '.') then
        nomeSom := nomeSom + '.wav';
    prMci.iniciaMidia(nomeSom, false);
end;

procedure TformConfigPrancha.b_stopClick(Sender: TObject);
begin
    prMci.terminaMidia;
end;

procedure TformConfigPrancha.b_okClick(Sender: TObject);
begin
    prMci.terminaMidia;
    destroiPranchaGraf (pranchaGraf);
    close;
end;

procedure TformConfigPrancha.b_playFalaClick(Sender: TObject);
var nomeSom: string;
begin
    nomeSom := combo_Fala.text;
    if (nomeSom <> '') and (copy (nomeSom, length(nomeSom)-3, 1) <> '.') then
        nomeSom := nomeSom + '.wav';
    prMci.iniciaMidia(nomeSom, false);
end;

procedure TformConfigPrancha.b_stopFalaClick(Sender: TObject);
begin
    prMci.terminaMidia;
end;

procedure TformConfigPrancha.b_recFalaClick(Sender: TObject);
var
    nomeSom: string;
begin
    prMci.terminaMidia;
    nomeSom := combo_fala.text;
    if formGravador.checaSom (nomeSom) then
        begin
            formGravador.combo_nomeArq.text := nomeSom;
            formGravador.showModal;
        end;
end;

procedure TformConfigPrancha.b_recClick(Sender: TObject);
var
    nomeSom: string;
begin
    prMci.terminaMidia;
    nomeSom := combo_som.text;
    if formGravador.checaSom (nomeSom) then
        begin
            formGravador.combo_nomeArq.text := nomeSom;
            formGravador.showModal;
        end;
end;

end.
