{---------------------------------------------------------------}
{
{    EASY BOARD - Editor and presenter of
{                 Boards for alternative communication
{
{    Module: Interactive execution of boards
{
{    Authors: Jos Antonio Borges     (NCE/UFRJ)
{             Miryam Bonadiu Pelosi    (LabAssistiva/UFRJ)
{             Julio Tadeu da Silveira  (NCE/UFRJ)
{
{    Version: 1.1 in april/2015
{
{----------------------------------------------------------------}

unit prExecuta;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, jpeg, mmSystem, shellApi, printers, minireg,
  dateUtils;

type
  TFormExecuta = class(TForm)
    TimerVarredura: TTimer;
    TimerHisterese: TTimer;
    TimerSerializacao: TTimer;
    Panel1: TPanel;
    Shape1: TShape;

    procedure FormActivate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TimerVarreduraTimer(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure Panel1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Panel1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TimerHistereseTimer(Sender: TObject);
    procedure TimerSerializacaoTimer(Sender: TObject);

  private
    { Private declarations }

    procedure exibePrancha (indPrancha: integer);
    function selPrancha (qualPrancha: string;
                                  var pranchaSel: integer;
                                  avisaErro: boolean): boolean;
    procedure inibeVarredura;
    procedure liberaVarredura;
    procedure inibeClick;
    procedure liberaClick;
    procedure trataAutoClick (autoAperta: boolean);
    function executaSocorro: boolean;
    procedure executaArquivo (executar: string);
    procedure printForm(frm: TForm);

  public
    { Public declarations }
    function selInicial (qualPrancha: string): boolean;
  end;

var
  FormExecuta: TFormExecuta;

implementation

uses prVars, prSobre, prGraficos, prMci, prSint, prDicasExec, prTobii;

var
    pranchaExecuta: TPranchaGraf;

    pranchaInicial, pranchaAtual, pranchaRecente, pranchaAnterior: integer;
    itemAtual: integer;
    salvaTempoVarredura: integer;
    varreduraInibida: boolean;
    clickInibido: boolean;
    tempoMouseDown: TDateTime;
    aExibirQuandoPossivel: integer;
    ultimoItemNoCursor: integer;
    tempoPermanencia: integer;
    ultimoItemClicado: integer;

const
    A_MESMA = 99999;

{$R *.dfm}

{--------------------------------------------------------}

function TFormExecuta.selInicial (qualPrancha: string): boolean;
begin
    result := selPrancha (qualPrancha, pranchaAtual, true);

    pranchaInicial := pranchaAtual;
    pranchaRecente := pranchaAtual;
    pranchaAnterior := pranchaAtual;

    salvaTempoVarredura := 0;
    TimerVarredura.Interval := projeto.tempoVarredura;
    TimerHisterese.Interval := projeto.tempoHisterese;
    ultimoItemClicado := -1;    ///////////////////////////////////////////

    if projeto.acionaTobii then
        prtobii.ativaTobii(0);
end;

{--------------------------------------------------------}

procedure TFormExecuta.FormActivate(Sender: TObject);
begin
    aExibirQuandoPossivel := pranchaAtual;
    inibeVarredura;
    prSint.sintAbre (projeto.falaAutomatica);

    if WindowState <> wsMaximized then
        WindowState := wsMaximized
    else
        FormResize(Sender);

    TimerSerializacao.Enabled := true;
end;

{--------------------------------------------------------}

procedure TFormExecuta.FormResize(Sender: TObject);
begin
    aExibirQuandoPossivel := pranchaAtual;

    { nota: o click s ser liberado pelo processo de varredura/histerese }
end;

{--------------------------------------------------------}

procedure TFormExecuta.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    threadTobiiAtiva := false;
    inibeVarredura;
    limpaFilaSom;
    prSint.sintFecha;
    destroiPranchaGraf (pranchaExecuta);

    aExibirQuandoPossivel := -1;
    TimerSerializacao.Enabled := false;

    WindowState := wsNormal;
    action := caHide;
end;

{--------------------------------------------------------}

function executaProgEx (nomeProg, nomeDir, nomeArq: string;
                        modoJanela: integer): THandle;
var
    pprog, pdir, pparam: array [0..511] of char;
    erro: tHandle;
    p: integer;
    paramExtra: string;

begin
    nomeProg := trim (nomeProg);
    if copy (nomeProg, 1, 1) = '"' then
        begin
            p := pos ('"', copy (nomeProg, 2, 999)) + 1;
            if p <> 0 then
                begin
                    paramExtra := trim (copy (nomeProg, p+1, 999));
                    delete (nomeProg, p+1, 999);
                    if paramExtra <> '' then
                        nomeArq := paramExtra + ' ' + nomeArq;
                end;
        end;

    strPcopy (pprog, nomeProg);
    strPCopy (pdir, nomeDir);
    strPcopy (pparam, nomeArq);

    erro := ShellExecute(FormExecuta.Handle, 'open', pprog, pparam, pdir, modoJanela);
    executaProgEx := erro;
end;

{--------------------------------------------------------}

function esperaProgAtivar: HWnd;   // embora sncrona, esta rotina tem tempo limitado
var n: integer;
    jan: HWnd;
begin
    n := 50;
    repeat
        sleep (100);
        dec(n);
        jan := getForegroundWindow;
    until (n = 0) or (jan <> FormExecuta.Handle);
    result := jan;
end;

{--------------------------------------------------------}

procedure TFormExecuta.executaArquivo (executar: string);
var i: integer;
    WindPlace : TWindowPlacement;
    WndHnd : THandle;

begin
    for i := 1 to 100 do
         begin
             if filaSomVazia then break;
             sleep (100);
         end;

    if executaProgEx (executar, '.', '', SW_SHOWNORMAL) > 32 then
        begin
            esperaProgAtivar;
            sleep (4000);
            WndHnd := GetForegroundWindow;
            Windows.GetWindowPlacement(WndHnd, @windPlace);
            with windPlace.rcNormalPosition do
                SetCursorPos(right-18, top+12);
        end;
end;

{---------------------------------------------------------}

function extraiIncrDoNome (var nomePrancha: string): integer;
var p, v, erro: integer;
begin
    result := 0;

    p := pos ('+', nomePrancha);
    if p > 1 then
        begin
             val (copy (nomePrancha, p+1, 999), v, erro);
             if erro = 0 then
                 begin
                     result := v;
                     nomePrancha := copy (nomePrancha, 1, p-1);
                     exit;
                 end;
        end;

    p := pos ('-', nomePrancha);
    if p > 1 then
        begin
             val (copy (nomePrancha, p+1, 999), v, erro);
             if erro = 0 then
                 begin
                     result := -v;
                     nomePrancha := copy (nomePrancha, 1, p-1);
                     exit;
                 end;
        end
end;

{---------------------------------------------------------}

function TFormExecuta.selPrancha (qualPrancha: string;
                                  var pranchaSel: integer;
                                  avisaErro: boolean): boolean;
var i: integer;
    incr: integer;

label erro;

begin
    qualPrancha := trim (qualPrancha);

    incr := extraiIncrDoNome (qualPrancha);
    if ansiUpperCase (qualPrancha) = '#ANTERIOR' then
        begin
            i := pranchaAnterior + incr;
            if (i < 0) or (i >= projeto.nPranchas) then goto erro;
            pranchaSel := i;
            result := true;
            exit;
        end;

    for i := 0 to projeto.nPranchas-1 do
        if ansiUpperCase (trim(projeto.pranchas[i].nome)) =
           ansiUpperCase (qualPrancha) then
        begin
            pranchaSel := i + incr;
            itemAtual := 0;
            result := true;
            exit;
        end;

erro:
    if avisaErro then
        showMessage ('Board ' + qualPrancha + ' not found');
    result := false;
end;

{--------------------------------------------------------}

procedure ajustaNomeArquivo (var nome: string);
var i: integer;
begin
    repeat
        i := pos ('*', nome);
        if i <> 0 then
            delete (nome, i, 1);
    until i = 0;
    repeat
        i := pos ('?', nome);
        if i <> 0 then
            delete (nome, i, 1);
    until i = 0;
end;

{--------------------------------------------------------}

procedure TFormExecuta.exibePrancha (indPrancha: integer);
var i: integer;
begin
    destroiPranchaGraf (pranchaExecuta);

    with projeto.pranchas[indPrancha] do
        begin
            criaPranchaGraf (pranchaExecuta, titulo, nFigx, nfigy, cor,
                             itens, panel1);
            for i := 0 to nitens-1 do
                setCallbackItemGraf(pranchaExecuta.gp_item[i],
                    panel1MouseDown, panel1MouseUp, i);

            pranchaAtual := indPrancha;
            itemAtual := 0;

            enfileiraSom (som);
            enfileiraSom (fala);
        end;
end;

{--------------------------------------------------------}

procedure TFormExecuta.Panel1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
    tempoMouseDown := now;
    if clickInibido then exit;

    inibeVarredura;
end;

{--------------------------------------------------------}

procedure TFormExecuta.Panel1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var i: integer;
    desvio: string;
begin
    if clickInibido then exit;  // evita reentrncia no VCL

    if executaSocorro then exit;

    inibeClick;
    i := (Sender as TComponent).tag;

    with projeto.pranchas[pranchaAtual].itens[i] do
        begin
            inibeVarredura;

            enfileiraSom (som);
            enfileiraSom (fala);

            if projeto.falaAutomatica and (prSint.sintet <> NIL) then
                enfileiraFala (rotulo);

            if executar <> '' then    // inicia a execuo assncrona
                executaArquivo (executar);

            desvio := link;
            if desvio = '' then
                desvio := projeto.pranchas[pranchaAtual].linkDefault;

            aExibirQuandoPossivel := A_MESMA;

            pranchaAnterior := pranchaRecente;
            pranchaRecente  := pranchaAtual;

            if desvio <> '' then
                if selPrancha (desvio, pranchaAtual, true) then
                    aExibirQuandoPossivel := pranchaAtual;
        end;
end;

{--------------------------------------------------------}

function TFormExecuta.executaSocorro: boolean;
var pranchaCandidata: integer;
begin
    if (projeto.tempoSocorro = 0) or
       (MilliSecondsBetween (tempoMouseDown, now) < projeto.tempoSocorro)  then
        begin
            result := false;
            exit;
        end;

    pranchaAtual := pranchaInicial;
    if (projeto.pranchaSocorro <> '') and
        selPrancha(projeto.pranchaSocorro, pranchaCandidata, true) then
            pranchaAtual := pranchaCandidata;

    aExibirQuandoPossivel := pranchaAtual;
    result := true;
end;

{--------------------------------------------------------}

procedure TFormExecuta.trataAutoClick (autoAperta: boolean);
var novoItemNoCursor: integer;
    desvio: string;
begin
    if aExibirQuandoPossivel >= 0 then exit;

   novoItemNoCursor := descobreItemNoCursor (FormExecuta, pranchaExecuta);
   if novoItemNoCursor < 0 then
       begin
           Shape1.visible := false;
           exit;
       end;

   with pranchaExecuta.gp_item[novoItemNoCursor] do
       if alocado and (not clickInibido) then
           begin
               Shape1.Left := gpanel.Left-10;
               Shape1.Top  := gpanel.Top-10;
               Shape1.Width := gpanel.Width+20;
               Shape1.Height := gpanel.Height+20;
               Shape1.visible := true;
               pranchaExecuta.gp_label.SendToBack;
            end;

    if not autoAperta then exit;

    if (not filaSomVazia) or
       (novoItemNoCursor <> ultimoItemNoCursor) or
       (novoItemNoCursor = ultimoItemClicado) then   /////////////////////////////////////////
        begin
            tempoPermanencia := 0;
            ultimoItemNoCursor := novoItemNoCursor;
            screen.Cursor := crDefault;
        end
    else
        begin
            tempoPermanencia := tempoPermanencia + integer(TimerSerializacao.Interval);
            if tempoPermanencia >= (integer(TimerSerializacao.Interval) div 2) then
                screen.Cursor := crHandPoint;
            if (not clickInibido) and (tempoPermanencia > projeto.tempoAutoClique) then
                begin
                    inibeVarredura;
                    ultimoItemClicado := novoItemNoCursor;  //////////////////////////////////

                    with projeto.pranchas[pranchaAtual].itens[novoItemNoCursor] do
                        begin
                            enfileiraSom (som);
                            enfileiraSom (fala);
                        end;

                    desvio := projeto.pranchas[pranchaAtual].itens[novoItemNoCursor].link;
                    if desvio = '' then
                        desvio := projeto.pranchas[pranchaAtual].linkDefault;

                    aExibirQuandoPossivel := A_MESMA;
                    pranchaAnterior := pranchaRecente;
                    pranchaRecente  := pranchaAtual;

                    if desvio <> '' then
                        if selPrancha (desvio, pranchaAtual, true) then
                            begin
                                aExibirQuandoPossivel := pranchaAtual;
                                ultimoItemClicado := -1;    /////////////////////////////
                            end;

                    tempoPermanencia := 0;
                    ultimoItemNoCursor := -1;
                end;
        end;
end;

{--------------------------------------------------------}

procedure TFormExecuta.TimerSerializacaoTimer(Sender: TObject);
begin
    if getForegroundWindow <> FormExecuta.Handle then
        begin
            if not varreduraInibida then
                inibeVarredura;
            aExibirQuandoPossivel := pranchaAtual;
            exit;
        end;

    if TimerHisterese.Interval = 0 then
        trataAutoClick (projeto.tempoAutoClique <> 0);

    if filaSomVazia then
        if (aExibirQuandoPossivel >= 0) and
           (getForegroundWindow = FormExecuta.Handle)
           then
                begin
                    if aExibirQuandoPossivel <> A_MESMA then
                        exibePrancha (pranchaAtual);
                    liberaVarredura;
                    aExibirQuandoPossivel := -1;
                end;
end;

{--------------------------------------------------------}

procedure TFormExecuta.PrintForm(frm: TForm);
var
    bmp: TBitMap;
    x, y, WDPI, HDPI: Integer;
begin
    bmp := frm.GetFormImage;
    with Printer do
        begin
            Orientation := poLandscape;
            BeginDoc;
            HDPI := PageHeight div 8;
            WDPI := PageWidth div 8;
            x := PageWidth - Round(WDPI * 0.4); {0.4" margem direita}
            y := PageHeight - Round(HDPI * 0.5); {0.5" Altura do rodap}
            Canvas.StretchDraw(Rect(Round(HDPI * 0.3), Round(HDPI * 0.35), x, y), bmp);
            EndDoc;
        end;
    bmp.Free;
end;

{--------------------------------------------------------}

procedure TFormExecuta.FormKeyPress(Sender: TObject; var Key: Char);
begin
    inibeVarredura;

    case key of
       ' ':   if salvaTempoVarredura = 0 then
                  begin
                       salvaTempoVarredura := timerVarredura.interval;
                       timerVarredura.interval := 0;
                  end
              else
                  begin
                       timerVarredura.interval := salvaTempoVarredura;
                       salvaTempoVarredura := 0;
                  end;

       '+':   timerVarredura.Interval := timerVarredura.interval + 1000;

       '-':   if timerVarredura.Interval > 1000 then
                  timerVarredura.interval := timerVarredura.interval-1000
              else
                  timerVarredura.interval := 0;

       '*':   begin
                  exibePrancha (pranchaAtual);
                  clickInibido := false;
              end;

       '/':   begin
                  if selPrancha (pranchaEscolhida, pranchaAtual, true) then
                      exibePrancha(pranchaAtual);
                  clickInibido := false;
              end;

        ^p:   begin
                  printForm (FormExecuta);
              end;

        '?':   formDicasExec.ShowModal;
    end;

    liberaVarredura;
end;

{--------------------------------------------------------}

procedure TFormExecuta.TimerVarreduraTimer(Sender: TObject);
var max: integer;
begin
   with pranchaExecuta do
       begin
           Shape1.visible := false;
           max := 1000;
           while not (gp_item[itemAtual].alocado and
                      gp_item[itemAtual].gpanel.visible and
                      (max <> 0)
                      ) do
               begin
                   itemAtual := (itemAtual + 1) mod length(gp_item);
                   max := max - 1;
               end;

           if max = 0 then
               begin
                   showMessage ('Execution error: this board contains unallocated items');
                   close;
               end;

           with pranchaExecuta.gp_item[itemAtual] do
               if alocado then
               begin
                   Shape1.Left := gpanel.Left-10;
                   Shape1.Top  := gpanel.Top-10;
                   Shape1.Width := gpanel.Width+20;
                   Shape1.Height := gpanel.Height+20;
                   Shape1.visible := true;
                   pranchaExecuta.gp_label.SendToBack;

                   SetCursorPos(shape1.Left+ClientOrigin.X+56, shape1.top+ClientOrigin.Y+56);
                   if filaSomVazia then
                       enfileiraFala (glabel.Caption);
                end;
       end;

   itemAtual := (itemAtual + 1) mod length(pranchaExecuta.gp_item);
end;

{--------------------------------------------------------}

procedure TFormExecuta.inibeVarredura;
begin
    varreduraInibida := true;
    TimerVarredura.Enabled := false;
    shape1.visible := false;
end;

{--------------------------------------------------------}

procedure TFormExecuta.liberaVarredura;
begin
    if timerHisterese.Interval <> 0 then
        if timerHisterese.Interval > tempoMouseDown-now+100 then
             begin
                 timerHisterese.Enabled := true;
                 screen.cursor := crHSplit;
                 exit;
             end;

    TimerVarredura.Enabled := TimerVarredura.Interval > 0;
    liberaClick;
    varreduraInibida := false;
end;

{--------------------------------------------------------}

procedure TFormExecuta.TimerHistereseTimer(Sender: TObject);
begin
    timerHisterese.enabled := false;
    screen.cursor := crDefault;
    TimerVarredura.Enabled := TimerVarredura.Interval > 0;
    liberaClick;
    varreduraInibida := false;
end;

{--------------------------------------------------------}

procedure TFormExecuta.inibeClick;
begin
    clickInibido := true;
    Screen.Cursor := crHourGlass;
end;

{--------------------------------------------------------}

procedure TFormExecuta.liberaClick;
begin
    clickInibido := false;
    Screen.Cursor := crDefault;
end;

{--------------------------------------------------------}

end.

