{---------------------------------------------------------------}
{
{    Easy Board - Editor and Presenter of
{                 Boards for alternative communication
{
{    Module: Export and Import
{
{    Authors: Jos Antonio Borges     (NCE/UFRJ)
{             Miryam Bonadiu Pelosi    (LabAssistiva/UFRJ)
{             Julio Tadeu da Silveira  (NCE/UFRJ)
{
{    Version 1.1 in april/2015
{
{----------------------------------------------------------------}

unit prExporta;

interface
uses Windows, sysUtils, shellapi, Forms, Dialogs, Controls, Classes;

procedure exportaPasta (dirPranchas, dirDestino: string);
procedure importaPasta (dirOrigem, dirPranchas: string);

implementation

uses prVars, prArq;

{---------------------------------------------------------}

procedure exportaPasta (dirPranchas, dirDestino: string);
var
    dir, dir1, dir2: string;
    FileAttrs: integer;
    sr: TSearchRec;
    sl: TStringList;
    i: integer;
    save_Cursor:TCursor;

begin
    Save_Cursor := Screen.Cursor;

    getDir (0, dir);

    dir2 := '%$';
    {$I-} chdir (dirDestino);  {$I+}
    if ioresult = 0 then
        getDir (0, dir2);

    dir1 := '%$';
    {$I-} chdir (dirPranchas);  {$I+}
    if ioresult = 0 then
        getDir (0, dir1);

    if dir1 = dir2 then
         begin
             showMessage ('Files can not be copied on themselves');
             chdir (dir);
             exit;
         end;

    sl := TStringList.Create;

    fileAttrs := faArchive;
    if FindFirst('*.*', FileAttrs, sr) = 0 then
        begin
            repeat
                if (sr.Attr and FileAttrs) <> 0 then
                    sl.add (sr.Name);
            until FindNext(sr) <> 0;
            FindClose(sr);
        end;

    screen.Cursor := crHourGlass;
    for i := 0 to sl.count-1 do
         if not copyFile (pchar(dirPranchas + '\' + sl[i]),
                          pchar(dirDestino  + '\' + sl[i]), false) then
            if MessageDlg('Error writing file ' + sl[i] + '. Continue?',
                    mtConfirmation, [mbYes, mbNo], 0) = mrNo then
                         break;
    sl.Free;
    screen.Cursor := Save_Cursor;
    chdir (dir);

    showMessage ('End of copy');
end;

{---------------------------------------------------------}

procedure importaPasta (dirOrigem, dirPranchas: string);
var
    dir, dir1, dir2: string;
    FileAttrs: integer;
    sr: TSearchRec;
    sl, slDest: TStringList;
    i: integer;
    save_Cursor:TCursor;
    s: string;

    function existePrancha (s: string): boolean;
    var n: integer;

    begin
        result := false;
        if (copy (s, 1, 1) <> '[') or (copy (s, length(s), 1) <> ']') then
            exit;

        delete (s, 1, 1);
        delete (s, length(s), 1);
        s := ansiUpperCase(s);

        for n := 0 to projeto.npranchas-1 do
           begin
               if ansiUpperCase (projeto.pranchas[n].nome) = s then
                  begin
                      result := true;
                      exit;
                  end;
           end;
    end;

begin
    Save_Cursor := Screen.Cursor;

    getDir (0, dir);

    dir1 := '%$';              // usa getdir para equalizar os dois nomes
    {$I-} chdir (dirPranchas);  {$I+}
    if ioresult = 0 then
        getDir (0, dir1);

    dir2 := '%$';
    {$I-} chdir (dirOrigem);  {$I+}
    if ioresult = 0 then
        getDir (0, dir2);

    if dir1 = dir2 then
         begin
             showMessage ('Files can not be copied on themselves');
             chdir (dir);
             exit;
         end;

    sl := TStringList.Create;

    fileAttrs := faArchive;
    if FindFirst('*.*', FileAttrs, sr) = 0 then
        begin
            repeat
                if (sr.Attr and FileAttrs) <> 0 then
                    sl.add (sr.Name);
            until FindNext(sr) <> 0;
            FindClose(sr);
        end;

    screen.Cursor := crHourGlass;
    for i := 0 to sl.count-1 do
         if upperCase(sl[i]) <> 'PRANCHAS.PRA' then
             if not copyFile (pchar(dirOrigem   + '\' + sl[i]),
                              pchar(dirPranchas + '\' + sl[i]), false) then
                 if MessageDlg('Error writing file ' + sl[i] + '. Continue?',
                        mtConfirmation, [mbYes, mbNo], 0) = mrNo then
                         break;

    sl.loadFromFile (dirOrigem + '\pranchas.pra');

    slDest := TStringList.Create;
    if FileExists(dirPranchas + '\pranchas.pra') then
        slDest.loadFromFile (dirPranchas + '\pranchas.pra');

    for i := 0 to sl.Count-1 do
         begin
             s := trim(sl[i]);
             if (s = '') or (s[1] = ';') or (upperCase(s) = '[CONFIG]') then
                 continue;
             if s[1] = '[' then break;
         end;

    slDest.Add('');
    slDest.Add(';---------------------------------------');
    slDest.Add('');

    for i := i to sl.Count-1 do
         begin
             s := sl[i];
             if copy (s, 1, 1) = '[' then
                 begin
                     while existePrancha (s) do
                         begin
                             showMessage ('Board with duplicated name: ' + s);
                             insert ('-', s, length(s)-1);
                         end;
                 end;

             slDest.Add(s);
         end;

    sl.Free;
    slDest.SaveToFile(dirPranchas + '\pranchas.pra');
    slDest.Free;

    screen.Cursor := Save_Cursor;
    chdir (dir);

    carregaPranchas(dirPranchas);

    showMessage ('End model coupling');
end;

end.
