{---------------------------------------------------------------}
{
{    EASY BOARD - Editor and Presenter of
{                 Boards for alternative communication
{
{    Module: Drawing elements on screen
{
{    Authors: Jos Antonio Borges     (NCE/UFRJ)
{             Miryam Bonadiu Pelosi    (LabAssistiva/UFRJ)
{             Julio Tadeu da Silveira  (NCE/UFRJ)
{
{    Version 1.1 in abril/2015
{
{----------------------------------------------------------------}

unit prGraficos;

interface

uses
  Consts,     // verso customizada para portugus
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, prVars, dialogs;


procedure calcBBoxItem (qualFig: integer;
                        dxPrancha, dyPrancha: integer;
                        dyTitulo: integer;
                        nx, ny: integer;
                        desloca: boolean;
                        var bBox: TRect);

procedure criaItemGraf (var itemGraf: TItemGraf;
                        bbox: TRect;
                        imagem: string;
                        semImagem: boolean;
                        texto: string;
                        semTexto: boolean;
                        posTexto: TPosVertical;
                        dimTexto: integer;
                        cor: TColor;
                        dimMoldura: TDimMoldura;
                        PainelPai: TPanel);

procedure destroiItemGraf (var itemGraf: TItemGraf);

procedure setCallbackItemGraf (var itemGraf: TItemGraf;
                               onMouseDownCallback: TMouseEvent;
                               onMouseUpCallback: TMouseEvent;
                               ATag: integer);

procedure criaPranchaGraf (var pranchaGraf: TPranchaGraf;
                           titulo: string;
                           nx, ny: integer;
                           cor: TColor;
                           itens: TItemArray;
                           PainelPai: TPanel);

procedure destroiPranchaGraf (var pranchaGraf: TPranchaGraf);

function veOndeClicou (Sender: TObject): integer;

function descobreItemNoCursor (form: TForm; pranchaGrafica: TPranchaGraf): integer;

implementation

const
    percentSeparaItemX = 10;
    percentSeparaItemY = 10;
    percentMargemFigura  = 3;

    
procedure calcBBoxItem (qualFig: integer;
                        dxPrancha, dyPrancha: integer;
                        dyTitulo: integer;
                        nx, ny: integer;
                        desloca: boolean;
                        var bBox: TRect);
var
    x, y: integer;
    dxSepara,  dySepara: integer;
    dxItens, dyItens: integer;
    dxDesloc: integer;

begin
    x := qualFig mod nx;
    y := qualFig div nx;

    dxSepara := Round((dxPrancha * percentSeparaItemX / 100) / (nx+1));
    dySepara := Round((dyPrancha * percentSeparaItemY / 100) /   ny  );

    if dyTitulo = 0 then
        dyTitulo := dySepara;
    dxItens := dxPrancha - dxSepara;
    dyItens := dyPrancha - dyTitulo;

    with bBox do
    begin
        Top    := dyTitulo + Round(y * dyItens / ny);
        Left   := dySepara + Round(x * dxItens / nx);
        Bottom := dyTitulo + Round((y+1) * dyItens / ny) - dySepara;
        Right  := dySepara + Round((x+1) * dxItens / nx) - dySepara;

        if desloca then
            begin
                dxDesloc := (right - left) div 2;
                left := left + dxDesloc;
                right := right + dxDesloc;
            end;
    end;
end;

procedure criaItemGraf (var itemGraf: TItemGraf;
                        bbox: TRect;
                        imagem: string;
                        semImagem: boolean;
                        texto: string;
                        semTexto: boolean;
                        posTexto: TPosVertical;
                        dimTexto: integer;
                        cor: TColor;
                        dimMoldura: TdimMoldura;
                        PainelPai: TPanel);

var
    margem: integer;
    im: string;
    p: integer;
    largMoldura: integer;

        function descobreImagem (imagem, texto: string): string;
        var im: string;
        begin
            im := imagem;
            if FileExists(imagem) then im := imagem
            else
            if FileExists(imagem+'.jpg') then im := imagem+'.jpg'
            else
            if FileExists(texto+'.jpg') then im := texto+'.jpg';
            repeat
                 p := pos('?', im);
                 if p <> 0 then delete (im, p, 1);
            until p = 0;
            repeat
                 p := pos('*', im);
                 if p <> 0 then delete (im, p, 1);
            until p = 0;
            result := im;
        end;

begin
    with itemGraf do
        begin
            alocado := true;
            invisivel := semImagem and semTexto;

            gpanel := TPanel.Create (PainelPai);
            with gpanel do
            begin
                Left := bbox.Left;
                Top := bbox.Top;
                Width := bbox.Right-bbox.Left+1;
                Height := bbox.Bottom-bbox.Top+1;
                Color := cor;

                Tag := 0;
                Anchors := [akLeft, akTop, akRight, akBottom];
                BevelOuter := bvNone;
                Parent := PainelPai;
                Visible := true;
            end;

            gmoldura := TShape.Create(gpanel);
            largMoldura := 0;
            if dimMoldura <> sem then
                with gmoldura do
                begin
                    top := 0;
                    left := 0;
                    width := gpanel.width;
                    height := gpanel.height;
                    pen.Color := clBlack;
                    case dimMoldura of
                     // sem:    largMoldura := 0;
                        fina:   largMoldura := ord (dimMoldura);
                        media:  largMoldura := ord (dimMoldura)*width div 100;
                        grossa: largMoldura := ord (dimMoldura)*width div 50;
                    end;
                    pen.Width := largMoldura;

                    brush.Style := bsClear;
                    parent := gpanel;
                    visible := true;
                end;

            glabel := TLabel.Create (gpanel);
            if not semTexto then
                with glabel do
                begin
                    Left := largMoldura;
                    // Font.Size := gpanel.Height div (2*TEXTO_MEDIO-dimTexto);       //**************************************
                    Font.Size := trunc(dimTexto * painelPai.Height / Screen.Height);

                    Width := gpanel.Width-2*largMoldura;
                    Height := -Font.Height + gpanel.Height div 60;

                    case posTexto of
                        baixo: Top := gpanel.Height - Height - largMoldura;
                        meio:  Top := (gpanel.Height - Height) div 2;
                        cima:  Top := largMoldura;
                    end;

                    Alignment := taCenter;
                    Anchors := [akLeft, akRight, akBottom];
                    AutoSize := False;
                    Caption := Texto;
                    Font.Charset := ANSI_CHARSET;
                    Font.Color := clWindowText;

                    Font.Name := 'Arial Narrow';
                    Font.Style := [fsBold];
                    ParentFont := False;
                    Parent := gpanel;
                    Visible := true;
                end;

            gimage := TImage.Create (gpanel);
            if not semImagem then
                with gimage do
                begin
                    margem := (gpanel.Height * percentMargemFigura) div 100;
                    Left := margem + largMoldura;
                    Width := gpanel.Width - 2*(margem+largMoldura);
                    Top := margem + largMoldura;
                    Height := gpanel.Height - 2*(margem+largMoldura);

                    case posTexto of
                        baixo: Height := Height - glabel.Height;
                        cima:
                          begin
                            Top := Top + glabel.Height;
                            Height := Height - glabel.Height;
                          end;
                    end;

                    Anchors := [akLeft, akTop, akRight, akBottom];
                    Center := True;
                    Proportional := True;
                    Stretch := True;
                    Parent := gpanel;

                    im := descobreImagem (imagem, texto);
                    if im <> '' then
                        Picture.LoadFromFile(im);
                    Visible := true;
                end;

            glabel.BringToFront;

            if semImagem and semTexto then
                itemGraf.gpanel.visible := false;
        end;
end;

procedure destroiItemGraf (var itemGraf: TItemGraf);
begin
    with itemGraf do
        begin
            if not alocado then exit;

            gmoldura.visible := false;
            glabel.visible := false;
            gimage.visible := false;
            gpanel.visible := false;

            gpanel.OnMouseUp := NIL;
            gimage.OnMouseUp := NIL;
            glabel.OnMouseUp := NIL;

            gpanel.OnMouseDown := NIL;
            gimage.OnMouseDown := NIL;
            glabel.OnMouseDown := NIL;

            gpanel.parent := NIL;

            gmoldura.free;
            glabel.free;
            gimage.free;
            gpanel.free;

            alocado := false;
        end;

end;

procedure setCallbackItemGraf (var itemGraf: TItemGraf;
                               onMouseDownCallback: TMouseEvent;
                               onMouseUpCallback: TMouseEvent;
                               ATag: integer);
begin
    if not itemGraf.alocado then exit;

    itemGraf.gpanel.tag := ATag;
    itemGraf.gimage.tag := ATag;
    itemGraf.glabel.tag := ATag;

    itemGraf.gpanel.OnMouseDown := onMouseDownCallback;
    itemGraf.gimage.OnMouseDown := onMouseDownCallback;
    itemGraf.glabel.OnMouseDown := onMouseDownCallback;

    itemGraf.gpanel.OnMouseUp := OnMouseUpCallback;
    itemGraf.gimage.OnMouseUp := OnMouseUpCallback;
    itemGraf.glabel.OnMouseUp := OnMouseUpCallback;
end;

procedure criaPranchaGraf (var pranchaGraf: TPranchaGraf;
                           titulo: string;
                           nx, ny: integer;
                           cor: TColor;
                           itens: TItemArray;
                           PainelPai: TPanel);
var
    bbox: TRect;
    i: integer;
    dytitulo: integer;
    desloca: boolean;

begin
    destroiPranchaGraf (pranchaGraf);

    painelPai.Color := cor;

    pranchaGraf.gp_label := tLabel.Create(painelPai);


    with pranchaGraf.gp_label do
        begin
            // Font.Size := painelPai.Height div 18;                       //**************************************************************
            Font.Size := trunc(TEXTO_TITULO * painelPai.Height / Screen.Height);
            if titulo = '' then
                dyTitulo := 0
            else
                dyTitulo := -Font.Height + painelPai.Height div 30;

            Top := 0;
            Left := 0;
            Width := painelPai.Width;
            Height := dytitulo;

            Alignment := taCenter;
            Anchors := [akLeft, akRight, akBottom];
            AutoSize := False;
            Font.Charset := ANSI_CHARSET;
            Font.Color := clWindowText;
            Font.Name := 'Arial Narrow';
            Font.Style := [fsBold];

            Caption := titulo;
            Parent := painelPai;
            visible := true;
        end;

    with pranchaGraf do
        begin
            setLength (gp_item, nx*ny);

            for i := nx*ny-1 downto 0 do
                begin
                    if length(itens) > i then
                        desloca := itens[i].deslocHoriz
                    else
                        desloca := false;

                    calcBBoxItem (i, painelPai.Width, painelPai.Height,
                                  dytitulo, nx, ny, desloca, bBox);

                    if length(itens) > i then
                        with itens[i] do
                            criaItemGraf (gp_item[i], bbox,
                                  figura, semFigura, rotulo, semRotulo,
                                  posTexto, dimTexto, cor, dimMoldura, PainelPai);
                end;

            alocada := true;
        end;
end;

procedure destroiPranchaGraf (var pranchaGraf: TPranchaGraf);
var
    i: integer;
begin
    with pranchaGraf do
        begin
            if not alocada then exit;

            gp_label.visible := false;
            gp_label.free;                  

            for i := 0 to length(gp_item)-1 do
                destroiItemGraf(gp_item[i]);

            alocada := false;
        end;
end;

function veOndeClicou (Sender: TObject): integer;
begin
    if Sender is TImage then
        result := (Sender as TImage).Tag
    else
    if Sender is TLabel then
        result := (Sender as TLabel).Tag
    else
    if Sender is TPanel then
        result := (Sender as TPanel).Tag
    else
        result := -1;
end;

function descobreItemNoCursor (form: TForm; pranchaGrafica: TPranchaGraf): integer;
var pt: TPoint;
    i: integer;
begin
    result := -1;

    GetCursorPos(pt);
    pt.X := pt.X - form.ClientOrigin.X;
    pt.Y := pt.Y - form.ClientOrigin.Y;

    for i := 0 to length(pranchaGrafica.gp_item)-1 do
        with pranchaGrafica.gp_item[i] do
            begin
                if invisivel then continue;
                if (pt.X >= gpanel.Left) and
                   (pt.X <= gpanel.Left+gpanel.Width) and
                   (pt.Y >= gpanel.Top) and
                   (pt.Y <= gpanel.Top+gpanel.Height) then
                       begin
                           result := i;
                           exit;
                       end;
            end;
end;

end.

