{---------------------------------------------------------------}
{
{    EASY BOARD  - Editor and Presenter of
{                  Boards for alternative communication
{
{    Mdule: Sound record
{
{    Authors: Jos Antonio Borges     (NCE/UFRJ)
{             Miryam Bonadiu Pelosi    (LabAssistiva/UFRJ)
{             Julio Tadeu da Silveira  (NCE/UFRJ)
{
{    Version 1.1 in april/2015
{
{----------------------------------------------------------------}

unit prGravador;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, ExtCtrls, dvgrav, mmSystem, Buttons;

type
    TIPO_CORTE = (CORTA_INICIO, CORTA_FIM, ZERA);

type
  TFormGravador = class(TForm)
    TimerGrav: TTimer;
    Image1: TImage;
    b_cortaInicio: TButton;
    b_cortaFim: TButton;
    b_maisVol50: TButton;
    b_menosVolume: TButton;
    b_play: TBitBtn;
    b_record: TBitBtn;
    b_salva: TButton;
    b_limpaSom: TButton;
    b_stop: TBitBtn;
    b_fadeOut: TButton;
    l_duracao: TLabel;
    b_carrega: TButton;
    combo_nomeArq: TComboBox;
    l_status: TLabel;
    procedure b_tocarClick(Sender: TObject);
    procedure b_gravarClick(Sender: TObject);
    procedure TimerGravTimer(Sender: TObject);
    procedure tocar1Click(Sender: TObject);
    procedure Gravar1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure b_cortaInicioClick(Sender: TObject);
    procedure b_cortaFimClick(Sender: TObject);
    procedure b_menosVolumeClick(Sender: TObject);
    procedure b_maisVol50Click(Sender: TObject);
    procedure b_salvaClick(Sender: TObject);
    procedure b_stopClick(Sender: TObject);
    procedure b_carregaClick(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure b_fadeOutClick(Sender: TObject);
    procedure combo_nomeArqSelect(Sender: TObject);
    procedure b_limpaSomClick(Sender: TObject);
  private
    { Private declarations }
    procedure EixoGrafico;
    procedure MostraGrafico;
    procedure volumeSom (percentual: real);
  public
    { Public declarations }
    function checaSom (var nomeSom: string): boolean;
  end;

var
  FormGravador: TFormGravador;

implementation

{$R *.dfm}

const
    SOMMAX = 10000000;
type
    VETSOM = packed array [-22..SOMMAX] of smallInt;

var
    dir: string;
    gravando: boolean;
    psom: ^VETSOM;

    quantAmostras: integer;
    FVelocidade: integer;
    FBitsporAmostra: integer;
    FCanais: integer;


procedure adicionaArquivos (sl: TStrings; extensao: string);
var FileAttrs: integer;
    sr: TSearchRec;
begin
    fileAttrs := faArchive;
    if FindFirst('*.' + extensao, FileAttrs, sr) = 0 then
        begin
            repeat
                if (sr.Attr and FileAttrs) <> 0 then
                    sl.add (sr.Name);
            until FindNext(sr) <> 0;
            FindClose(sr);
        end;
end;

procedure TFormGravador.EixoGrafico;
var rect: TRect;
begin
   with image1 do
       begin
           canvas.pen.color := clRed;
           rect.Left := 0;
           rect.Top := 0;
           rect.Right := width;
           rect.Bottom := height;
           canvas.FillRect (rect);
           Canvas.MoveTo(0, height div 2);
           canvas.LineTo(width, height div 2);
           canvas.pen.color := clBlack;
       end;
end;

procedure TFormGravador.MostraGrafico;
var
    dx, ampl: real;
    i: integer;
    x: real;
begin
    EixoGrafico;

    ampl := image1.height / 65000.0;
    if quantAmostras = 0 then
        dx := 1
    else
        dx := (image1.width-1) / quantAmostras;

    x := 0;
    image1.canvas.moveTo (trunc(x), image1.height-1-trunc (32768 * ampl));

    for i := 0 to quantAmostras-1 do
        begin
            image1.canvas.lineto (trunc(x), image1.height-1-trunc ((psom^[i]+32768) * ampl));
            x := x + dx;
        end;

    l_duracao.caption := intToStr(quantAmostras div 22050) + ' seg.';
end;

procedure TFormGravador.FormActivate(Sender: TObject);
begin
    getDir (0, dir);

    l_status.Caption := '';
    combo_nomeArq.Items.Clear;
    adicionaArquivos(combo_nomeArq.Items, 'wav');

    gravando := false;
    new (psom);
    application.ProcessMessages;

    eixoGrafico;
    b_carregaClick(Sender);
end;

procedure TFormGravador.FormDeactivate(Sender: TObject);
begin
    sndPlaySound(NIL, SND_ASYNC+SND_MEMORY);
    Dispose (psom);
end;

procedure genWavHdr (pvet: pchar; veloc: longint; bits, channels: word; size: longint);
const
    wavHdr: array [0..43] of byte = (
        $52, $49, $46, $46,    {'RIFF'}
        $ff, $ff, $ff, $ff,    {riff size}
        $57, $41, $56, $45, $66, $6d, $74, $20,    {'WAVEFMT '}
        $10, $00, $00, $00,    {hdr size}
        $01, $00, $01, $00, $11, $2b, $00, $00, $11, $2b, $00, $00, $01, $00, $08, $00,  {reg}
        $64, $61, $74, $61,    {'data'}
        $ff, $ff, $ff, $ff);   {data size}

var l: longint;
    p: pointer;
    lpFormat: PPCMWAVEFORMAT;

begin
    new (lpFormat);
    with lpFormat^, lpFormat^.wf do
        begin
            wFormatTag := WAVE_FORMAT_PCM;
            nSamplesPerSec := veloc;
            wBitsPerSample := bits;
            nChannels := channels;
            nBlockAlign := (wBitsPerSample div 8) * nChannels;
            nAvgBytesPerSec := nBlockAlign * nSamplesPerSec;
        end;

    p := @wavHdr[20];
    move (lpFormat^, p^, sizeof (lpFormat^));
    l := size + 36;
    p := @wavHdr[4];
    move (l, p^, sizeof (l));
    p := @wavHdr[40];
    move (size, p^, sizeof (size));

    move (wavHdr, pvet^, sizeof (wavHdr));
    dispose (lpFormat);
end;

procedure TFormGravador.b_carregaClick(Sender: TObject);
const
    MAXBUFSIZE = 65532;
var
    soundBuffer: pchar;
    f: integer;
    erroIO: integer;
    chunkSize: integer;
    faltaLer: longint;
    lidos, aLer: integer;
    lpFormat: PPCMWAVEFORMAT;
    achou: boolean;
    pvet: pchar;
    i: integer;

    smint: array[0..MAXBUFSIZE*2-1] of smallInt;


label fim;

begin
    f := FileOpen(combo_nomeArq.text, fmOpenRead or fmShareDenyNone);
    if f < 0 then exit;

    getMem (soundBuffer, MAXBUFSIZE);

    erroIO := 1;   {'Sound format unknowed'}

    lidos := fileRead (f, soundBuffer^, 12);     { checa cabealho RIFF }
    if lidos < 12 then goto fim;
    if strlicomp (soundBuffer, pchar('RIFF'), 4) <> 0 then goto fim;

    lidos := fileRead (f, soundBuffer^, 8);      { checa fmt }
    if lidos < 8 then goto fim;
    if strlicomp (soundBuffer, 'fmt ', 4) <> 0 then goto fim;

    move (soundBuffer[4], chunkSize, 4);
    lidos := fileRead (f, soundBuffer^, chunkSize);   { checa fmt }
    if chunkSize <> lidos then goto fim;

    lpFormat := @soundBuffer[0];

    if lpFormat^.wf.wFormatTag <> WAVE_FORMAT_PCM then
        goto fim;    // s processo PCM

    with lpFormat^, lpFormat^.wf do
        begin
            FVelocidade := nSamplesPerSec;
            FBitsporAmostra := wBitsPerSample;
            FCanais := nChannels;
        end;

    pvet := @(psom^[0]);

    repeat
        lidos := fileRead (f, soundBuffer^, 8);     { ignora chunks at data }
        if lidos < 8 then goto fim;
        achou := strlicomp (soundBuffer, 'data', 4) = 0;
        move (soundBuffer[4], chunkSize, 4);
        if not achou then
            begin
               lidos := fileRead (f, soundBuffer^, chunkSize);     { checa fmt }
               if lidos <> chunkSize then goto fim;
            end;
    until achou;

    quantAmostras := chunkSize div (FCanais * (FBitsporAmostra div 8));
    if quantAmostras = 0 then
        begin
            erroIO := 2;   {'sound file is empty'}
            goto fim;
        end;

    if quantAmostras > SOMMAX then
        begin
            erroIO := 3;   {'this sound is too large to be edited here'}
            goto fim;
        end;

    if (FBitsporAmostra > 16) or
       ((FVelocidade <> 11025) and (FVelocidade <> 22050) and (FVelocidade <> 44100)) then
        begin
            erroIO := 4;    {'The sampling mode of sound is incompatible with this editor.'+^m^j+ }
                            {'Use another sound editor to adjust it.'}
            goto fim;
        end;

    // processamento das amostras do arquivo

    erroIO := 0;

    faltaLer := chunkSize;
    while faltaLer > 0 do
        begin
            if faltaLer <= MAXBUFSIZE then
                aLer := faltaLer
            else
                aLer := MAXBUFSIZE;
            faltaLer := faltaLer - aLer;

            lidos := fileRead (f, soundBuffer^, aLer);
            if lidos <> aLer then
                erroIO := 5;    {'File seems corrupted.'}
            if lidos = 0 then break;

            if (FBitsporAmostra = 16) and (FCanais = 1) then
                begin
                    move (soundBuffer^, pvet^, lidos);
                    inc (pvet, lidos);
                end
            else
            if (FBitsporAmostra = 16) and (FCanais = 2) then
                begin
                    move (soundBuffer^, smint, lidos);
                    for i := 0 to (lidos div 2) - 1 do
                        smint[i] := (smint[i*2] + smint[i*2]+1) div 2;
                    move (smint, pvet^, lidos div 2);
                    inc (pvet, lidos div 2);
                end
            else
            if (FBitsporAmostra = 8) and (FCanais = 1) then
                begin
                    for i := 0 to lidos-1 do
                        smint[i] := (ord(soundBuffer[i]) - 128) shl 8;
                    move (smint, pvet^, lidos*2);
                    inc (pvet, lidos*2);
                end
            else
            // if (FBitsporAmostra = 8) and (FCanais = 2) then
                begin
                    for i := 0 to lidos-1 do
                        smint[i] := (ord(soundBuffer[i]) - 128) shl 8;
                    for i := 0 to (lidos div 2) - 1 do
                        smint[i] := (smint[i*2] + smint[i*2]+1) div 2;
                    move (smint, pvet^, lidos);
                    inc (pvet, lidos);
                end;
        end;

    if FVelocidade = 11025 then
        begin
            for i := quantAmostras-1 downto 0 do
                begin
                    psom^[i*2] := psom^[i];
                    psom^[i*2+1] := psom^[i];
                end;
            quantAmostras := quantAmostras * 2;
        end
    else
    if FVelocidade = 44100 then
        begin
            for i := 0 to quantAmostras-1 do
                psom^[i] := (psom^[i*2] + psom^[i*2+1]) div 2;
            quantAmostras := quantAmostras div 2;
        end;

fim:
    case erroIO of
        0: ;
        1: showMessage ('Sound recording format not recognized');
        2: showMessage ('Empty sound file');
        3: showMessage ('Sound too large to be edited here');
        4: showMessage (' sampling mode of sound is incompatible with this sound editor.'+^m^j+
                        'Use another sound editor to adjust it.');
        5: showMessage ('File seems corrupted.');
    end;

    freeMem (soundBuffer, MAXBUFSIZE);
    fileClose (f);

    MostraGrafico;
end;

procedure TFormGravador.b_tocarClick(Sender: TObject);
begin
    if gravando then
        b_gravarClick(Sender);

    genWavHdr (@psom^[-22], 22050, 16, 1, quantAmostras*2);

    sndPlaySound(@psom^[-22], SND_ASYNC+SND_MEMORY);
end;

procedure TFormGravador.b_gravarClick(Sender: TObject);
begin
    sndPlaySound(NIL, snd_Async);
    if gravando then
        begin
            gravando := false;
            TimerGrav.Enabled := false;
            monitoraGravacao;
            terminaGravacao;

            l_status.Caption := '';
            b_carregaClick(Sender);
        end
    else
    if preparaGravacao(combo_nomeArq.text, 22050, 16, 1, 4, 4096) = 0 then
        begin
            TimerGrav.Enabled := true;
            l_status.Caption := 'Recording';
            gravando := true;
            iniciaGravacao;
        end;
end;

procedure TFormGravador.b_stopClick(Sender: TObject);
begin
    if gravando then
        b_gravarClick(Sender)
    else
        sndPlaySound (NIL, snd_async);
end;

procedure TFormGravador.TimerGravTimer(Sender: TObject);
begin
    monitoraGravacao;
end;

procedure TFormGravador.tocar1Click(Sender: TObject);
begin
    b_tocarClick(Sender);
end;

procedure TFormGravador.Gravar1Click(Sender: TObject);
begin
   b_gravarClick(Sender);
end;

procedure TFormGravador.b_cortaInicioClick(Sender: TObject);
var i: integer;
begin
    for i := 0 to quantAmostras - 1000-1 do
        psom^[i] := psom^[i+1000];

    mostraGrafico;
end;

procedure TFormGravador.b_cortaFimClick(Sender: TObject);
begin
    quantAmostras := quantAmostras - 1000;

    mostraGrafico;
end;

procedure TFormGravador.volumeSom (percentual: real);
var
    i: integer;
    n: integer;

begin
    for i := 0 to quantAmostras-1 do
        begin
            n := trunc(psom^[i] * percentual);
            if n < -32768 then
                n := -32768
            else
            if n > 32767 then
               n := 32767;
            psom^[i] := n;
        end;

    mostraGrafico;
end;

procedure TFormGravador.b_menosVolumeClick(Sender: TObject);
begin
    volumeSom (0.85);
end;

procedure TFormGravador.b_maisVol50Click(Sender: TObject);
begin
    volumeSom (1.20);
end;

procedure TFormGravador.b_fadeOutClick(Sender: TObject);
var
    i, x: integer;
    ampl: real;
begin
    x := quantAmostras - 4 * 22050;   { 4 segundos }
    if x < 0 then x := 0;

    for i := x to quantAmostras -1 do
        begin
            ampl := (quantAmostras - i) / (quantAmostras - x);
            psom^[i] := trunc(psom^[i] * ampl);
        end;

    MostraGrafico;
end;

procedure TFormGravador.b_salvaClick(Sender: TObject);
const
    MAXBUFSIZE = 65532;
var
    f: integer;
    aGravar, faltaGravar: longint;
    gravados: integer;
    pvet: pchar;

label fim;

begin
    genWavHdr (@psom^[-22], 22050, 16, 1, quantAmostras*2);

    f := FileCreate(combo_nomeArq.text);
    if f < 0 then
        begin
            showMessage ('Error while opening the space for recording');
            exit;
        end;

    pvet := @(psom^[-22]);
    faltaGravar := quantAmostras * 2 + 44;

    while faltaGravar > 0 do
        begin
            if faltaGravar <= MAXBUFSIZE then
                aGravar := faltaGravar
            else
                aGravar := MAXBUFSIZE;

            gravados := FileWrite (f, pvet^, aGravar);
            pvet := pvet + aGravar;
            faltaGravar := faltaGravar - aGravar;

            if gravados <> aGravar then
                begin
                    showMessage ('Write error on file.');
                    break;
                end;
        end;

    fileClose (f);

    if faltaGravar = 0 then
        begin
            combo_nomeArq.Items.Clear;
            adicionaArquivos(combo_nomeArq.Items, 'wav');
            showMessage ('File saved');
            close;
        end;
end;

function TFormGravador.checaSom (var nomeSom: string): boolean;
var ext: string;
begin
    checaSom := false;

    if nomeSom = '' then
        begin
            showMessage ('Please, enter the name of the sound file to record.');
            exit;
        end;

    if copy (nomeSom, length(nomeSom)-3, 1) <> '.' then
        nomeSom := nomeSom + '.wav';

    ext := upperCase(copy (nomeSom, length(nomeSom)-3, 4));
    if ext <> '.WAV' then
        begin
            showMessage ('This version only edit files .WAV');
            exit;
        end;

    if FileExists(nomeSom) then
        begin
            if MessageDlg('File exist, confirm change?',
                mtWarning, mbOKCancel, 0) <> mrOK then
                    exit;
        end;

    checaSom := true;
end;

procedure TFormGravador.combo_nomeArqSelect(Sender: TObject);
begin
    b_carregaClick(Sender);
end;

procedure TFormGravador.b_limpaSomClick(Sender: TObject);
begin
    quantAmostras := 0;
    mostraGrafico;
end;

end.


