{---------------------------------------------------------------}
{
{    PRANCHA FCIL - Editor e Apresentador de
{                    Pranchas para Comunicao Alternativa
{
{    Mdulo de tratamento das mdias
{
{    Autores: Jos Antonio Borges     (NCE/UFRJ)
{             Miryam Bonadiu Pelosi    (LabAssistiva/UFRJ)
{             Julio Tadeu da Silveira  (NCE/UFRJ)
{
{    Verso 1.1 em abril/2015
{
{----------------------------------------------------------------}

unit prMci;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, ExtCtrls, dvgrav, mmSystem, Buttons;

procedure iniciaMidia (nomeArq: string; espera: boolean);
function midiaTocando: boolean;
procedure terminaMidia;

procedure tocaSom (som: string);

procedure limpaFilaSom;
procedure enfileiraSom(nomeSom: string);
procedure enfileiraFala(fala: string);
procedure movimentaFilaSom;   // controlada por um timer de 100 ms
function filaSomVazia: boolean;

implementation

uses
     prSint, prvars;

var
    erroMci: string;
    retornoMci: string;
    filaSom: TStringList;
    midiaAberta: boolean;

const
    SINTETICO = #$01;
    
{--------------------------------------------------------}

function enviaComandoMCI (s: string): integer;
var p, retorno: array [0..255] of char;
    erro: integer;
begin
    strPcopy (p, s);
    erro := mciSendString (p, retorno, 255, 0);
    if erro <> 0 then
        mciGetErrorString (erro, p, 255)
    else
        erroMci := '';
    retornoMci := strPas (retorno);
    result := erro;
end;

{--------------------------------------------------------}

procedure iniciaMidia (nomeArq: string; espera: boolean);
var dir: string;
begin
    nomeArq := trim(nomeArq);
    if nomeArq = '' then exit;

    if pos ('\', nomeArq) = 0 then
        begin
            getDir (0, dir);
            if dir [length(dir)] <> '\' then dir := dir + '\';
            nomeArq := dir + nomeArq;
        end;

    nomeArq := '"' + nomeArq + '"';

    enviaComandoMci ('close midiaPrancha');
    enviaComandoMci ('open ' + nomeArq + ' alias midiaPrancha');
    if espera then
        enviaComandoMci ('play midiaPrancha wait')
    else
        enviaComandoMci ('play midiaPrancha');

    midiaAberta := true;
end;

{--------------------------------------------------------}

procedure terminaMidia;
begin
    if not midiaAberta then
        exit;
    enviaComandoMci ('stop midiaPrancha');
    enviaComandoMci ('close midiaPrancha');
    midiaAberta := false;
end;

{--------------------------------------------------------}

function midiaTocando: boolean;
begin
    enviaComandoMci ('status midiaPrancha mode');
    result := upperCase (retornoMCI) = 'PLAYING';
end;

{--------------------------------------------------------}

procedure tocaSom (som: string);
var somOriginal: string;
begin
    somOriginal := som;
    if som <> '' then
        begin
            if som[1] = SINTETICO then
                begin
                    delete (som, 1, 1);
                    sintFala (som);
                    exit;
                end;

            if (som <> '') and (copy (som, length(som)-3, 1) <> '.') then
                som := som + '.wav';
            if pos ('\', som) = 0 then
                som := GetCurrentDir + '\' + som;
            if fileExists (som) then
                begin
                     terminaMidia;
                     iniciaMidia(som, false);
                end
            else
                if projeto.SintetNaoGravados then
                     sintFala(somOriginal);
        end;
end;

{--------------------------------------------------------}

procedure limpaFilaSom;
begin
    if sintAtivo then sintCala;
    if midiaAberta then
        terminaMidia;

    filaSom.clear;
end;

{--------------------------------------------------------}

procedure enfileiraSom(nomeSom: string);
begin
    if nomeSom <> '' then
        filaSom.add (nomeSom);
end;

{--------------------------------------------------------}

procedure enfileiraFala(fala: string);
begin
    if fala <> '' then
        filaSom.add (#$01 + fala)
    else
        sintCala;
end;

{--------------------------------------------------------}

procedure movimentaFilaSom;   // controlada por um timer de 100 ms
begin
    if sintAtivo then exit;
    if midiaAberta and midiaTocando then exit;

    if filaSom.count <> 0 then
        begin
            tocaSom (filaSom [0]);
            filaSom.delete (0);
        end;
end;

{--------------------------------------------------------}

function filaSomVazia: boolean;
begin
    movimentaFilaSom;
    result := not (
                sintAtivo or
                (midiaAberta and midiaTocando) or
                (filaSom.Count <> 0)
              );
end;

{--------------------------------------------------------}

begin
    filaSom := TStringList.Create;
end.

