{---------------------------------------------------------------}
{
{    Easy Board -       Editor and Presenter of
{                    Boards for alternative communication
{
{               Selection Module of initial board for execution
{
{    Authors: Jos Antonio Borges     (NCE/UFRJ)
{             Miryam Bonadiu Pelosi    (LabAssistiva/UFRJ)
{             Julio Tadeu da Silveira  (NCE/UFRJ)
{
{    Version 1.1: april/2015
{
{----------------------------------------------------------------}

unit prSelInic;

interface

uses
  Consts,     // customized version to english
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, FileCtrl, ComCtrls, ShellCtrls, minireg,
  Buttons, shellApi;

type
  TformSelInic = class(TForm)
    Label1: TLabel;
    Panel1: TPanel;
    b_criarProjeto: TButton;
    b_editar: TButton;
    b_imprimir: TButton;
    b_exportar: TButton;
    b_executar: TButton;
    rg_onde: TRadioGroup;
    Image1: TImage;
    b_configurar: TButton;
    b_renovar: TButton;
    l_pasta: TLabel;
    b_escolherPasta: TButton;
    b_salvar: TButton;
    b_sobre: TButton;
    b_importar: TButton;
    b_ajuda: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure Image1Click(Sender: TObject);
    procedure b_renovarClick(Sender: TObject);
    procedure b_criarProjetoClick(Sender: TObject);
    procedure b_escolherPastaClick(Sender: TObject);
    procedure b_exportarClick(Sender: TObject);
    procedure b_salvarClick(Sender: TObject);
    procedure b_editarClick(Sender: TObject);
    procedure b_imprimirClick(Sender: TObject);
    procedure b_configurarClick(Sender: TObject);
    procedure b_sobreClick(Sender: TObject);
    procedure b_executarClick(Sender: TObject);
    procedure b_importarClick(Sender: TObject);
    procedure b_ajudaClick(Sender: TObject);
  private
    { Private declarations }
     function selProjeto(dir: string; podeCriar: boolean): boolean;
     function defineDirBase: string;
    procedure ligaBotoes;
  public
    { Public declarations }
  end;

var
  formSelInic: TformSelInic;

implementation

uses prVars, prSobre, prArq, prExporta, prProg, prConfigExec,
     prSelImprime, prExecuta, prSelExecuta, prDicasExec;

var noInicio: boolean;

{$R *.dfm}

procedure TformSelInic.FormCreate(Sender: TObject);
begin
    label1.Caption := label1.Caption + versao;
    noInicio := true;

    pranchaEscolhida := '';
end;

procedure TformSelInic.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    if MessageDlg('Are you sure you want to end?  Already saved ?', mtConfirmation,
                    [mbYes, mbNo], 0) = mrYes then
        Action := caFree
    else
        Action := caNone;
end;

procedure TformSelInic.FormActivate(Sender: TObject);
begin
    b_renovarClick(Sender);

    if noInicio then
       begin
           AboutBox.ligaTimer;
           AboutBox.ShowModal;
           noInicio := false;
       end;
end;

function TformSelInic.defineDirBase: string;
var dir: string;
const
    SearchTree = 'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders\';

begin
    if rg_onde.ItemIndex = 0 then
        begin
            regGetString (HKEY_CURRENT_USER, SearchTree+'\Personal', dir);
            dir := dir + '\My Boards';
            {$I-} mkdir (dir);  {$I-}
            if ioresult <> 0 then;
        end
    else
    if rg_onde.ItemIndex = 1 then
        regGetString (HKEY_CURRENT_USER, SearchTree+'\Desktop', dir)
    else
    if rg_onde.ItemIndex = rg_onde.Items.Count-1 then
        begin
            regGetString (HKEY_CURRENT_USER, SearchTree+'\Desktop', dir);
            repeat
                delete (dir, length(dir), 1);
            until (dir <> '') and (dir[length(dir)] = '\');
            delete (dir, length(dir), 1);
        end
    else
        dir := rg_onde.Items[rg_onde.ItemIndex] [7]+ ':\';

    result := dir;
end;

function TformSelInic.selProjeto(dir: string; podeCriar: boolean): boolean;
const
    options_create: TSelectDirOpts = [sdAllowCreate, sdPerformCreate, sdPrompt];
    options_select: TSelectDirOpts = [];
var
    options: TSelectDirOpts;
    salvaDir: string;

begin
    if podeCriar then options := options_create
                 else options := options_select;

    if not podeCriar then
        showMessage ('On the next screen, double-click the folder you want to confirm');

    getDir (0, salvaDir);
    if SelectDirectory(dir, options, 0) then
        begin
            dirProjeto := dir;        // produce this global information
            l_pasta.Caption := dir;
            result := true;
        end
    else
        begin
            chdir (salvaDir);
            result := false;
        end;
end;

procedure TformSelInic.ligaBotoes;
begin
    b_editar.Enabled := true;
    b_salvar.Enabled := true;
    b_imprimir.Enabled := true;
    b_configurar.Enabled := true;
    b_exportar.Enabled := true;
    b_importar.Enabled := true;
    b_executar.Enabled := true;
end;

procedure TformSelInic.b_criarProjetoClick(Sender: TObject);
var novoNome, dir: string;
begin
    dir := defineDirBase;
    if not InputQuery('Enter the name of your project',
               'This will be the folder name to create inside of' + #$0d#$0a +
               dir, NovoNome) then
        exit;

    {$I-}
    dir := dir + '\' + NovoNome;
    mkdir (dir);
    {$I+}
    if ioresult <> 0 then
        begin
            showMessage ('The project folder can not be created, reuse or edit the name');
            selProjeto(dir, true);
        end
    else
        begin
            selProjeto(dir, false);
            setLength(projeto.pranchas, 0);
            projeto.nPranchas := 0;
        end;

    ligaBotoes;
    b_editarClick(Sender);
end;

procedure TformSelInic.b_escolherPastaClick(Sender: TObject);
var dirBase: string;
begin
    dirBase := defineDirBase;
    if selProjeto(dirBase, false) then   // preenche dirProjeto
        begin
            ligaBotoes;
            carregaPranchas(dirProjeto);
            b_editarClick(Sender);
        end;
end;

procedure TformSelInic.Image1Click(Sender: TObject);
begin
    AboutBox.ShowModal;
end;

procedure TformSelInic.b_renovarClick(Sender: TObject);
var
    letras: array [0..255] of char;
    i: integer;
    vol: array [0..30] of char;
    dummy: DWord;
    drive: string;
    salvaItem: integer;

begin
    salvaItem := rg_onde.ItemIndex;

    rg_onde.Items.Clear;
    rg_onde.Items.Add ('My Boards');
    rg_onde.Items.Add ('Desktop');

    GetLogicalDriveStrings(255, letras);
    i := 0;
    while (letras[i] <> #0) do
        begin
            drive := '';
            repeat
                drive := drive + letras[i];
                i := i + 1;
            until letras[i] = #0;

            if GetDriveType(pchar(drive)) = DRIVE_REMOVABLE then
                begin
                    vol := '';
                    GetVolumeInformation(pchar(drive), vol, 30, NIL, dummy, dummy, NIL, 30);
                    if vol = '' then
                        rg_onde.Items.Add ('Drive ' + drive[1])
                    else
                        rg_onde.Items.Add ('Drive ' + drive[1] + ' [' + vol + ']');
                end;

            i := i + 1;
        end;

    rg_onde.Items.Add ('Other folder');
    rg_onde.ItemIndex := salvaItem;
end;

procedure TformSelInic.b_salvarClick(Sender: TObject);
begin
    if dirProjeto = '' then
        begin
            showMessage ('project folder was not set');
            exit;
        end;

    renameFile (dirProjeto + '\pranchas.pra', dirProjeto + '\pranchas.bak');
    salvaPranchas(dirProjeto, 'pranchas.pra');
end;

procedure TformSelInic.b_editarClick(Sender: TObject);
begin
    if dirProjeto = '' then
        begin
            showMessage ('project folder was not set');
            exit;
        end;

    formProg.show;
end;

procedure TformSelInic.b_imprimirClick(Sender: TObject);
begin
    if dirProjeto = '' then
        begin
            showMessage ('You did not select any folder, can not print');
            exit;
        end;

    FormSelImprime.ShowModal;
end;

procedure TformSelInic.b_configurarClick(Sender: TObject);
begin
   FormConfigExec.ShowModal;
end;

procedure TformSelInic.b_sobreClick(Sender: TObject);
begin
    AboutBox.ShowModal;
end;

procedure TformSelInic.b_executarClick(Sender: TObject);
var dirBase: string;
begin
    if dirProjeto = '' then
        begin
            dirBase := defineDirBase;
            if selProjeto(dirBase, false) then   // preenche dirProjeto
                carregaPranchas(dirProjeto);
        end;

    if projeto.nPranchas = 0 then
        begin
            showMessage ('no boards in this project');
            dirProjeto := '';
            exit;
        end;

    ligaBotoes;

    pranchaEscolhida := projeto.pranchaInicial;
    if pranchaEscolhida = '' then
        begin
            formSelExecuta.showModal;
            pranchaEscolhida := FormSelExecuta.combo_pranchaInicial.Text;
        end;

    if pranchaEscolhida <> '' then
        begin
             formExecuta.selInicial(pranchaEscolhida);
             formExecuta.showModal;
        end;
end;

procedure TformSelInic.b_exportarClick(Sender: TObject);
var salva: string;
    dirDestino, novaPasta: string;
begin
    if dirProjeto = '' then
        begin
            showMessage ('You did not selected a project, can not export');
            exit;
        end;

    if not InputQuery('Exportation', 'What is the name of the boards folder that will be created?', novaPasta) then
        exit;

    showMessage ('Double click the place where the folder will be create');

    salva := dirProjeto;
    selProjeto(dirProjeto, true);
    if dirProjeto[length(dirProjeto)] <> '\' then
        dirProjeto := dirProjeto + '\';
    dirDestino := dirProjeto + novaPasta;
    dirProjeto := salva;
    l_pasta.Caption := dirProjeto;

    if DirectoryExists(dirDestino) then
        begin
            if MessageDlg('This folder already existed. Confirm possible destruction of content?',
                          mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
                exit;
        end
    else
        begin
            {$I-} mkdir (dirDestino);  {$I+}
            if ioresult <> 0 then
                 begin
                     showMessage ('Sorry. Could not create this folder to export.');
                     exit;
                 end;
        end;

    exportaPasta (dirProjeto, dirDestino);
end;

procedure TformSelInic.b_importarClick(Sender: TObject);
var dirOrigem, salva: string;
begin
    if dirProjeto = '' then
        begin
            showMessage ('Project folder not defined');
            exit;
        end;

    salva := dirProjeto;
    selProjeto(dirProjeto, false);
    dirOrigem := dirProjeto;

    dirProjeto := salva;
    l_pasta.Caption := dirProjeto;

    importaPasta (dirOrigem, dirProjeto);
    carregaPranchas(dirProjeto);
    b_editarClick(Sender);
end;

procedure TformSelInic.b_ajudaClick(Sender: TObject);
var Path: string;
begin
    Path := ExtractFilePath(Application.ExeName);
    if fileExists (Path + 'pranchaFacil.chm') then
        shellExecute (handle, 'OPEN', pchar(Path + 'pranchaFacil.chm'), NIL, '.', SW_SHOW)
    else
        messageDlg ('Help file not found', mtError, [mbok], 0);
end;

end.

