{---------------------------------------------------------------}
{
{    PRANCHA FCIL - Editor e Apresentador de
{                    Pranchas para Comunicao Alternativa
{
{    Mdulo de pr-visualizao 
{
{    Autores: Jos Antonio Borges     (NCE/UFRJ)
{             Miryam Bonadiu Pelosi    (LabAssistiva/UFRJ)
{             Julio Tadeu da Silveira  (NCE/UFRJ)
{
{    Verso 1.1 em abril/2015
{
{----------------------------------------------------------------}

unit prVisualiza;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Printers, mmSystem, jpeg, minireg;

type
  TformVisualiza = class(TForm)
    Panel1: TPanel;
    PrintDialog1: TPrintDialog;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Panel1Resize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure Panel1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Panel1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    procedure desenhaPrancha (ind: integer);
  public
    { Public declarations }
    procedure selPrancha (indPrancha: integer);
    procedure imprime;
    procedure maximiza;
    procedure geraJPG;
  end;

var
  formVisualiza: TformVisualiza;

implementation

{$R *.dfm}
uses prVars, prGraficos, prMci, prSint;

var
    pranchaVisualiza: TPranchaGraf;
    indAtual: integer;
    pranchaLiberada: boolean;

procedure TformVisualiza.selPrancha (indPrancha: integer);
begin
    indAtual := indPrancha;
end;

procedure TformVisualiza.FormActivate(Sender: TObject);
begin
    prSint.sintAbre (projeto.falaAutomatica);

    pranchaLiberada := true;
    Panel1Resize(Sender);
end;

procedure TformVisualiza.Panel1Resize(Sender: TObject);
var i: integer;
begin
    if not pranchaLiberada then exit;

    desenhaPrancha (indAtual);
    with projeto.pranchas[indAtual] do
        begin
            for i := 0 to nitens-1 do
                setCallbackItemGraf (pranchaVisualiza.gp_item[i],
                       panel1MouseDown, panel1MouseUp, i);
        end;
end;

procedure TformVisualiza.desenhaPrancha (ind: integer);
begin
    if ind < 0 then exit;
    indAtual := ind;
    with projeto.pranchas[indAtual] do
        begin
            criaPranchaGraf (pranchaVisualiza, titulo, nFigx, nfigy, cor,
                             itens, panel1);
            application.processMessages;
            if not somInibido then
                begin
                    showCursor (false);
                    tocaSom (som);
                    tocaSom (fala);
                    showCursor (true);
                end;
        end;
end;

procedure TformVisualiza.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
    sintFecha;
    pranchaLiberada := false;
    destroiPranchaGraf (pranchaVisualiza);
end;

procedure PrintForm(frm: TForm);
var
    bmp: TBitMap;
    x, y, WDPI, HDPI: Integer;
begin
    Screen.Cursor := crHourGlass;
    bmp := frm.GetFormImage;
    with Printer do
        begin
            Orientation := poLandscape;
            BeginDoc;
            HDPI := PageHeight div 8;
            WDPI := PageWidth div 8;
            x := PageWidth - Round(WDPI * 0.4); {0.4" margem direita}
            y := PageHeight - Round(HDPI * 0.5); {0.5" Altura do rodap}
            Canvas.StretchDraw(Rect(Round(HDPI * 0.3), Round(HDPI * 0.35), x, y), bmp);
            EndDoc;
        end;
    bmp.Free;
    Screen.Cursor := crDefault;
end;

procedure TformVisualiza.imprime;
var salva: TWindowState;
begin
    somInibido := true;
    salva := WindowState;

    if PrintDialog1.execute then
        begin
            WindowState := wsMaximized;
            application.ProcessMessages;

            desenhaPrancha (indAtual);
            sleep (2000);
            printForm (formVisualiza);
            application.ProcessMessages;
        end;

    WindowState := salva;
    somInibido := false;
end;

procedure TformVisualiza.GeraJPG;
var
    B: TBitmap;
    I: TJpegImage;
    dir, nomeJpg: string;

const
    SearchTree = 'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders\';

begin
    Screen.Cursor := crHourGlass;

    somInibido := true;
    desenhaPrancha (indAtual);
    somInibido := true;
    application.ProcessMessages;

    B := TBitmap.Create;
    try
        I := TJpegImage.Create;
        B := formVisualiza.GetFormImage;
        try
            I.Assign(B);
            regGetString (HKEY_CURRENT_USER, SearchTree+'\Desktop', dir);
            nomeJpg := projeto.pranchas[indAtual].nome+'.jpg';
            I.SaveToFile(dir + '\' + nomeJpg);
        finally
            I.Free;
        end;
    finally
        B.Free;
    end;

    Screen.Cursor := crDefault;
    application.processMessages;
    somInibido := false;

    showMessage (nomeJpg + ' gravado na rea de Trabalho');
end;

procedure TformVisualiza.maximiza;
begin
    somInibido := true;
    WindowState := wsMaximized;
    application.ProcessMessages;
    somInibido := false;
end;

procedure TformVisualiza.FormDeactivate(Sender: TObject);
begin
    destroiPranchaGraf (pranchaVisualiza);
end;

procedure TformVisualiza.Panel1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    beep;
end;

procedure TformVisualiza.Panel1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var i: integer;
begin
    i := veOndeClicou (Sender);
    if i < 0 then exit;

    with projeto.pranchas[indAtual].itens[i] do
        begin
            showCursor (false);
            if som  <> '' then tocaSom(som);
            if fala <> '' then tocaSom(fala);
            showCursor (true);

            if projeto.falaAutomatica and (sintet <> NIL) then
                begin
                    if trim(rotulo) <> '' then
                        begin
                            sintFala (rotulo);
                            while sintAtivo do waitMessage;
                        end;
                    sleep (300);
                end;

            if executar <> '' then
                showMessage ('Executaria ' + projeto.pranchas[indAtual].itens[i].executar);
            if link <> '' then
                showMessage ('Desviaria para ' + projeto.pranchas[indAtual].itens[i].link)
            else
            if projeto.pranchas[indAtual].linkDefault <> '' then
                showMessage ('Desviaria para ' + projeto.pranchas[indAtual].linkDefault)
            else
                showMessage ('Desvio no programado no item ' + intToStr(i));
        end;
end;

end.

